@@ -25,178 +25,68 @@ AomVarianceFnPtr svt_aom_mefn_ptr[BlockSizeS_ALL];
 
 void init_fn_ptr(void) {
 #if CONFIG_ENABLE_HIGH_BIT_DEPTH
-#define BFP0(BT, SDF, VF, VF_HBD_10, SVF, SDX4DF) \
-    svt_aom_mefn_ptr[BT].sdf       = SDF;         \
-    svt_aom_mefn_ptr[BT].vf        = VF;          \
-    svt_aom_mefn_ptr[BT].vf_hbd_10 = VF_HBD_10;   \
-    svt_aom_mefn_ptr[BT].svf       = SVF;         \
-    svt_aom_mefn_ptr[BT].sdx4df    = SDX4DF;
+#define BFP0(w, h)                                                                     \
+    svt_aom_mefn_ptr[BLOCK_##w##X##h].sdf       = svt_aom_sad##w##x##h;                \
+    svt_aom_mefn_ptr[BLOCK_##w##X##h].vf        = svt_aom_variance##w##x##h;           \
+    svt_aom_mefn_ptr[BLOCK_##w##X##h].vf_hbd_10 = svt_aom_highbd_10_variance##w##x##h; \
+    svt_aom_mefn_ptr[BLOCK_##w##X##h].svf       = svt_aom_sub_pixel_variance##w##x##h; \
+    svt_aom_mefn_ptr[BLOCK_##w##X##h].sdx4df    = svt_aom_sad##w##x##h##x4d;
 #else
-#define BFP0(BT, SDF, VF, VF_HBD_10, SVF, SDX4DF) \
-    svt_aom_mefn_ptr[BT].sdf    = SDF;            \
-    svt_aom_mefn_ptr[BT].vf     = VF;             \
-    svt_aom_mefn_ptr[BT].svf    = SVF;            \
-    svt_aom_mefn_ptr[BT].sdx4df = SDX4DF;
+#define BFP0(w, h)                                                                  \
+    svt_aom_mefn_ptr[BLOCK_##w##X##h].sdf    = svt_aom_sad##w##x##h;                \
+    svt_aom_mefn_ptr[BLOCK_##w##X##h].vf     = svt_aom_variance##w##x##h;           \
+    svt_aom_mefn_ptr[BLOCK_##w##X##h].svf    = svt_aom_sub_pixel_variance##w##x##h; \
+    svt_aom_mefn_ptr[BLOCK_##w##X##h].sdx4df = svt_aom_sad##w##x##h##x4d;
 #endif
-    BFP0(BLOCK_4X16,
-         svt_aom_sad4x16,
-         svt_aom_variance4x16,
-         svt_aom_highbd_10_variance4x16,
-         svt_aom_sub_pixel_variance4x16,
-         svt_aom_sad4x16x4d)
-    BFP0(BLOCK_16X4,
-         svt_aom_sad16x4,
-         svt_aom_variance16x4,
-         svt_aom_highbd_10_variance16x4,
-         svt_aom_sub_pixel_variance16x4,
-         svt_aom_sad16x4x4d)
-    BFP0(BLOCK_8X32,
-         svt_aom_sad8x32,
-         svt_aom_variance8x32,
-         svt_aom_highbd_10_variance8x32,
-         svt_aom_sub_pixel_variance8x32,
-         svt_aom_sad8x32x4d)
-    BFP0(BLOCK_32X8,
-         svt_aom_sad32x8,
-         svt_aom_variance32x8,
-         svt_aom_highbd_10_variance32x8,
-         svt_aom_sub_pixel_variance32x8,
-         svt_aom_sad32x8x4d)
-    BFP0(BLOCK_16X64,
-         svt_aom_sad16x64,
-         svt_aom_variance16x64,
-         svt_aom_highbd_10_variance16x64,
-         svt_aom_sub_pixel_variance16x64,
-         svt_aom_sad16x64x4d)
-    BFP0(BLOCK_64X16,
-         svt_aom_sad64x16,
-         svt_aom_variance64x16,
-         svt_aom_highbd_10_variance64x16,
-         svt_aom_sub_pixel_variance64x16,
-         svt_aom_sad64x16x4d)
-    BFP0(BLOCK_128X128,
-         svt_aom_sad128x128,
-         svt_aom_variance128x128,
-         svt_aom_highbd_10_variance128x128,
-         svt_aom_sub_pixel_variance128x128,
-         svt_aom_sad128x128x4d)
-    BFP0(BLOCK_128X64,
-         svt_aom_sad128x64,
-         svt_aom_variance128x64,
-         svt_aom_highbd_10_variance128x64,
-         svt_aom_sub_pixel_variance128x64,
-         svt_aom_sad128x64x4d)
-    BFP0(BLOCK_64X128,
-         svt_aom_sad64x128,
-         svt_aom_variance64x128,
-         svt_aom_highbd_10_variance64x128,
-         svt_aom_sub_pixel_variance64x128,
-         svt_aom_sad64x128x4d)
-    BFP0(BLOCK_32X16,
-         svt_aom_sad32x16,
-         svt_aom_variance32x16,
-         svt_aom_highbd_10_variance32x16,
-         svt_aom_sub_pixel_variance32x16,
-         svt_aom_sad32x16x4d)
-    BFP0(BLOCK_16X32,
-         svt_aom_sad16x32,
-         svt_aom_variance16x32,
-         svt_aom_highbd_10_variance16x32,
-         svt_aom_sub_pixel_variance16x32,
-         svt_aom_sad16x32x4d)
-    BFP0(BLOCK_64X32,
-         svt_aom_sad64x32,
-         svt_aom_variance64x32,
-         svt_aom_highbd_10_variance64x32,
-         svt_aom_sub_pixel_variance64x32,
-         svt_aom_sad64x32x4d)
-    BFP0(BLOCK_32X64,
-         svt_aom_sad32x64,
-         svt_aom_variance32x64,
-         svt_aom_highbd_10_variance32x64,
-         svt_aom_sub_pixel_variance32x64,
-         svt_aom_sad32x64x4d)
-    BFP0(BLOCK_32X32,
-         svt_aom_sad32x32,
-         svt_aom_variance32x32,
-         svt_aom_highbd_10_variance32x32,
-         svt_aom_sub_pixel_variance32x32,
-         svt_aom_sad32x32x4d)
-    BFP0(BLOCK_64X64,
-         svt_aom_sad64x64,
-         svt_aom_variance64x64,
-         svt_aom_highbd_10_variance64x64,
-         svt_aom_sub_pixel_variance64x64,
-         svt_aom_sad64x64x4d)
-    BFP0(BLOCK_16X16,
-         svt_aom_sad16x16,
-         svt_aom_variance16x16,
-         svt_aom_highbd_10_variance16x16,
-         svt_aom_sub_pixel_variance16x16,
-         svt_aom_sad16x16x4d)
-    BFP0(BLOCK_16X8,
-         svt_aom_sad16x8,
-         svt_aom_variance16x8,
-         svt_aom_highbd_10_variance16x8,
-         svt_aom_sub_pixel_variance16x8,
-         svt_aom_sad16x8x4d)
-    BFP0(BLOCK_8X16,
-         svt_aom_sad8x16,
-         svt_aom_variance8x16,
-         svt_aom_highbd_10_variance8x16,
-         svt_aom_sub_pixel_variance8x16,
-         svt_aom_sad8x16x4d)
-    BFP0(BLOCK_8X8,
-         svt_aom_sad8x8,
-         svt_aom_variance8x8,
-         svt_aom_highbd_10_variance8x8,
-         svt_aom_sub_pixel_variance8x8,
-         svt_aom_sad8x8x4d)
-    BFP0(BLOCK_8X4,
-         svt_aom_sad8x4,
-         svt_aom_variance8x4,
-         svt_aom_highbd_10_variance8x4,
-         svt_aom_sub_pixel_variance8x4,
-         svt_aom_sad8x4x4d)
-    BFP0(BLOCK_4X8,
-         svt_aom_sad4x8,
-         svt_aom_variance4x8,
-         svt_aom_highbd_10_variance4x8,
-         svt_aom_sub_pixel_variance4x8,
-         svt_aom_sad4x8x4d)
-    BFP0(BLOCK_4X4,
-         svt_aom_sad4x4,
-         svt_aom_variance4x4,
-         svt_aom_highbd_10_variance4x4,
-         svt_aom_sub_pixel_variance4x4,
-         svt_aom_sad4x4x4d)
+    BFP0(4, 16)
+    BFP0(16, 4)
+    BFP0(8, 32)
+    BFP0(32, 8)
+    BFP0(16, 64)
+    BFP0(64, 16)
+    BFP0(128, 128)
+    BFP0(128, 64)
+    BFP0(64, 128)
+    BFP0(32, 16)
+    BFP0(16, 32)
+    BFP0(64, 32)
+    BFP0(32, 64)
+    BFP0(32, 32)
+    BFP0(64, 64)
+    BFP0(16, 16)
+    BFP0(16, 8)
+    BFP0(8, 16)
+    BFP0(8, 8)
+    BFP0(8, 4)
+    BFP0(4, 8)
+    BFP0(4, 4)
 #if CONFIG_ENABLE_OBMC
-#define OBFP(BT, OSDF, OVF, OSVF)     \
-    svt_aom_mefn_ptr[BT].osdf = OSDF; \
-    svt_aom_mefn_ptr[BT].ovf  = OVF;  \
-    svt_aom_mefn_ptr[BT].osvf = OSVF;
-    OBFP(BLOCK_128X128, svt_aom_obmc_sad128x128, svt_aom_obmc_variance128x128, svt_aom_obmc_sub_pixel_variance128x128)
-    OBFP(BLOCK_128X64, svt_aom_obmc_sad128x64, svt_aom_obmc_variance128x64, svt_aom_obmc_sub_pixel_variance128x64)
-    OBFP(BLOCK_64X128, svt_aom_obmc_sad64x128, svt_aom_obmc_variance64x128, svt_aom_obmc_sub_pixel_variance64x128)
-    OBFP(BLOCK_64X64, svt_aom_obmc_sad64x64, svt_aom_obmc_variance64x64, svt_aom_obmc_sub_pixel_variance64x64)
-    OBFP(BLOCK_64X32, svt_aom_obmc_sad64x32, svt_aom_obmc_variance64x32, svt_aom_obmc_sub_pixel_variance64x32)
-    OBFP(BLOCK_32X64, svt_aom_obmc_sad32x64, svt_aom_obmc_variance32x64, svt_aom_obmc_sub_pixel_variance32x64)
-    OBFP(BLOCK_32X32, svt_aom_obmc_sad32x32, svt_aom_obmc_variance32x32, svt_aom_obmc_sub_pixel_variance32x32)
-    OBFP(BLOCK_32X16, svt_aom_obmc_sad32x16, svt_aom_obmc_variance32x16, svt_aom_obmc_sub_pixel_variance32x16)
-    OBFP(BLOCK_16X32, svt_aom_obmc_sad16x32, svt_aom_obmc_variance16x32, svt_aom_obmc_sub_pixel_variance16x32)
-    OBFP(BLOCK_16X16, svt_aom_obmc_sad16x16, svt_aom_obmc_variance16x16, svt_aom_obmc_sub_pixel_variance16x16)
-    OBFP(BLOCK_16X8, svt_aom_obmc_sad16x8, svt_aom_obmc_variance16x8, svt_aom_obmc_sub_pixel_variance16x8)
-    OBFP(BLOCK_8X16, svt_aom_obmc_sad8x16, svt_aom_obmc_variance8x16, svt_aom_obmc_sub_pixel_variance8x16)
-    OBFP(BLOCK_8X8, svt_aom_obmc_sad8x8, svt_aom_obmc_variance8x8, svt_aom_obmc_sub_pixel_variance8x8)
-    OBFP(BLOCK_4X8, svt_aom_obmc_sad4x8, svt_aom_obmc_variance4x8, svt_aom_obmc_sub_pixel_variance4x8)
-    OBFP(BLOCK_8X4, svt_aom_obmc_sad8x4, svt_aom_obmc_variance8x4, svt_aom_obmc_sub_pixel_variance8x4)
-    OBFP(BLOCK_4X4, svt_aom_obmc_sad4x4, svt_aom_obmc_variance4x4, svt_aom_obmc_sub_pixel_variance4x4)
-    OBFP(BLOCK_4X16, svt_aom_obmc_sad4x16, svt_aom_obmc_variance4x16, svt_aom_obmc_sub_pixel_variance4x16)
-    OBFP(BLOCK_16X4, svt_aom_obmc_sad16x4, svt_aom_obmc_variance16x4, svt_aom_obmc_sub_pixel_variance16x4)
-    OBFP(BLOCK_8X32, svt_aom_obmc_sad8x32, svt_aom_obmc_variance8x32, svt_aom_obmc_sub_pixel_variance8x32)
-    OBFP(BLOCK_32X8, svt_aom_obmc_sad32x8, svt_aom_obmc_variance32x8, svt_aom_obmc_sub_pixel_variance32x8)
-    OBFP(BLOCK_16X64, svt_aom_obmc_sad16x64, svt_aom_obmc_variance16x64, svt_aom_obmc_sub_pixel_variance16x64)
-    OBFP(BLOCK_64X16, svt_aom_obmc_sad64x16, svt_aom_obmc_variance64x16, svt_aom_obmc_sub_pixel_variance64x16)
+#define OBFP(w, h)                                                           \
+    svt_aom_mefn_ptr[BLOCK_##w##X##h].osdf = svt_aom_obmc_sad##w##x##h;      \
+    svt_aom_mefn_ptr[BLOCK_##w##X##h].ovf  = svt_aom_obmc_variance##w##x##h; \
+    svt_aom_mefn_ptr[BLOCK_##w##X##h].osvf = svt_aom_obmc_sub_pixel_variance##w##x##h;
+    OBFP(128, 128)
+    OBFP(128, 64)
+    OBFP(64, 128)
+    OBFP(64, 64)
+    OBFP(64, 32)
+    OBFP(32, 64)
+    OBFP(32, 32)
+    OBFP(32, 16)
+    OBFP(16, 32)
+    OBFP(16, 16)
+    OBFP(16, 8)
+    OBFP(8, 16)
+    OBFP(8, 8)
+    OBFP(4, 8)
+    OBFP(8, 4)
+    OBFP(4, 4)
+    OBFP(4, 16)
+    OBFP(16, 4)
+    OBFP(8, 32)
+    OBFP(32, 8)
+    OBFP(16, 64)
+    OBFP(64, 16)
 #endif
 }
 
@@ -243,11 +133,11 @@ static int mvsad_err_cost_light(const Mv *mv, const Mv *ref) {
     const uint32_t mv_rate    = 1296 + (factor * (absmvdiffx + absmvdiffy));
     return mv_rate;
 }
-int svt_aom_mv_err_cost(const Mv *mv, const Mv *ref, const int *mvjcost, int *mvcost[2], int error_per_bit) {
+int svt_aom_mv_err_cost(const Mv *mv, const Mv *ref, const int *mvjcost, const int *mvcost[2], int error_per_bit) {
     if (mvcost) {
         const Mv diff = (Mv){{mv->x - ref->x, mv->y - ref->y}};
         return (int)ROUND_POWER_OF_TWO_64(
-            (int64_t)svt_mv_cost(&diff, mvjcost, (const int *const *)mvcost) * error_per_bit,
+            (int64_t)svt_mv_cost(&diff, mvjcost, mvcost) * error_per_bit,
             RDDIV_BITS + AV1_PROB_COST_SHIFT - RD_EPB_SHIFT + PIXEL_TRANSFORM_ERROR_SCALE);
     }
     return 0;
@@ -714,6 +604,7 @@ static int full_pixel_exhaustive(PictureControlSet *pcs, IntraBcContext *x, cons
     return bestsme;
 }
 
+#if CONFIG_ENABLE_OBMC
 static int get_obmc_mvpred_var(const IntraBcContext *x, const int32_t *wsrc, const int32_t *mask, const Mv *best_mv,
                                const Mv *center_mv, const AomVarianceFnPtr *vfp, int use_mvcost, int is_second) {
     const struct Buf2D *in_what = (const struct Buf2D *)(&x->xdplane[0].pre[is_second]);
@@ -765,7 +656,7 @@ static int obmc_refining_search_sad(const IntraBcContext *x, const int32_t *wsrc
     return best_sad;
 }
 
-int svt_av1_obmc_full_pixel_search(ModeDecisionContext *ctx, IntraBcContext *x, Mv *mvp_full, int sadpb,
+int svt_av1_obmc_full_pixel_search(struct ModeDecisionContext *ctx, IntraBcContext *x, const Mv *mvp_full, int sadpb,
                                    const AomVarianceFnPtr *fn_ptr, const Mv *ref_mv, Mv *dst_mv, int is_second) {
     // obmc_full_pixel_diamond does not provide BDR gain on 360p
     const int32_t *wsrc         = ctx->wsrc_buf;
@@ -782,7 +673,9 @@ int svt_av1_obmc_full_pixel_search(ModeDecisionContext *ctx, IntraBcContext *x,
 
     return thissme;
 }
+#endif
 
+#if CONFIG_ENABLE_OBMC
 static INLINE void set_subpel_mv_search_range(const MvLimits *mv_limits, int *col_min, int *col_max, int *row_min,
                                               int *row_max, const Mv *ref_mv) {
     const int max_mv = MAX_FULL_PEL_VAL * 8;
@@ -815,7 +708,7 @@ static const Mv search_step_table[12] = {
 static unsigned int setup_obmc_center_error(const int32_t *mask, const Mv *bestmv, const Mv *ref_mv, int error_per_bit,
                                             const AomVarianceFnPtr *vfp, const int32_t *const wsrc,
                                             const uint8_t *const y, int y_stride, int offset, int *mvjcost,
-                                            int *mvcost[2], unsigned int *sse1,
+                                            const int *mvcost[2], unsigned int *sse1,
                                             uint8_t use_low_precision_cost_estimation, int *distortion) {
     unsigned int besterr;
     besterr     = vfp->ovf(y + offset, y_stride, wsrc, mask, sse1);
@@ -940,7 +833,7 @@ static unsigned int upsampled_setup_obmc_center_error(MacroBlockD *xd, const Av1
                                                       int mi_col, const int32_t *mask, const Mv *bestmv,
                                                       const Mv *ref_mv, int error_per_bit, const AomVarianceFnPtr *vfp,
                                                       const int32_t *const wsrc, const uint8_t *const y, int y_stride,
-                                                      int w, int h, int offset, int *mvjcost, int *mvcost[2],
+                                                      int w, int h, int offset, int *mvjcost, const int *mvcost[2],
                                                       unsigned int *sse1, int *distortion,
                                                       uint8_t use_low_precision_cost_estimation, int subpel_search) {
     unsigned int besterr = upsampled_obmc_pref_error(
@@ -960,11 +853,12 @@ static INLINE const uint8_t *pre(const uint8_t *buf, int stride, int r, int c) {
     return buf + offset;
 }
 
-int svt_av1_find_best_obmc_sub_pixel_tree_up(ModeDecisionContext *ctx, IntraBcContext *x, const AV1_COMMON *const cm,
-                                             int mi_row, int mi_col, Mv *bestmv, const Mv *ref_mv, int allow_hp,
-                                             int error_per_bit, const AomVarianceFnPtr *vfp, int forced_stop,
-                                             int iters_per_step, int *mvjcost, int *mvcost[2], int *distortion,
-                                             unsigned int *sse1, int is_second, int use_accurate_subpel_search) {
+int svt_av1_find_best_obmc_sub_pixel_tree_up(struct ModeDecisionContext *ctx, IntraBcContext *x,
+                                             const struct Av1Common *const cm, int mi_row, int mi_col, Mv *bestmv,
+                                             const Mv *ref_mv, int allow_hp, int error_per_bit,
+                                             const AomVarianceFnPtr *vfp, int forced_stop, int iters_per_step,
+                                             int *mvjcost, const int *mvcost[2], int *distortion, unsigned int *sse1,
+                                             int is_second, int use_accurate_subpel_search) {
     const int32_t                 *wsrc        = ctx->wsrc_buf;
     const int32_t                 *mask        = ctx->mask_buf;
     const int *const               z           = wsrc;
@@ -1130,6 +1024,7 @@ int svt_av1_find_best_obmc_sub_pixel_tree_up(ModeDecisionContext *ctx, IntraBcCo
 
     return besterr;
 }
+#endif
 
 int svt_av1_full_pixel_search(PictureControlSet *pcs, IntraBcContext *x, BlockSize bsize, Mv *mvp_full, int step_param,
                               int method, int run_mesh_search, int error_per_bit, int *cost_list, const Mv *ref_mv,
