@@ -187,8 +187,8 @@ void SvtAv1E2ETestFramework::init_test(TestVideoVector &test_vector) {
     //
     // Init handle
     //
-    return_error = svt_av1_enc_init_handle(
-        &av1enc_ctx_.enc_handle, &av1enc_ctx_, &av1enc_ctx_.enc_params);
+    return_error = svt_av1_enc_init_handle(&av1enc_ctx_.enc_handle,
+                                           &av1enc_ctx_.enc_params);
     ASSERT_EQ(return_error, EB_ErrorNone)
         << "svt_av1_enc_init_handle return error:" << return_error;
     ASSERT_NE(av1enc_ctx_.enc_handle, nullptr)
@@ -214,12 +214,12 @@ void SvtAv1E2ETestFramework::init_test(TestVideoVector &test_vector) {
     ASSERT_NE(av1enc_ctx_.output_stream_buffer, nullptr)
         << "Malloc memory for output_stream_buffer failed.";
     av1enc_ctx_.output_stream_buffer->p_buffer =
-        new uint8_t[EB_OUTPUTSTREAMBUFFERSIZE_MACRO(width * height)];
+        new uint8_t[BITSTREAM_BUFFER_SIZE(width * height)];
     ASSERT_NE(av1enc_ctx_.output_stream_buffer->p_buffer, nullptr)
         << "Malloc memory for output_stream_buffer->p_buffer failed.";
     av1enc_ctx_.output_stream_buffer->size = sizeof(EbBufferHeaderType);
     av1enc_ctx_.output_stream_buffer->n_alloc_len =
-        EB_OUTPUTSTREAMBUFFERSIZE_MACRO(width * height);
+        BITSTREAM_BUFFER_SIZE(width * height);
     av1enc_ctx_.output_stream_buffer->p_app_private = nullptr;
     av1enc_ctx_.output_stream_buffer->pic_type = EB_AV1_INVALID_PICTURE;
     av1enc_ctx_.output_stream_buffer->metadata = nullptr;
@@ -251,14 +251,7 @@ void SvtAv1E2ETestFramework::init_test(TestVideoVector &test_vector) {
     ASSERT_EQ(return_error, EB_ErrorNone)
         << "svt_av1_enc_init return error:" << return_error;
 
-#if TILES_PARALLEL
-    Bool has_tiles = (Bool)(av1enc_ctx_.enc_params.tile_columns ||
-                            av1enc_ctx_.enc_params.tile_rows);
-#else
-    Bool has_tiles = (Bool)FALSE;
-#endif
-    obu_frame_header_size_ =
-        has_tiles ? OBU_FRAME_HEADER_SIZE + 1 : OBU_FRAME_HEADER_SIZE;
+    obu_frame_header_size_ = OBU_FRAME_HEADER_SIZE;
 
     // create reference decoder if required.
     if (enable_decoder) {
@@ -271,7 +264,7 @@ void SvtAv1E2ETestFramework::init_test(TestVideoVector &test_vector) {
     // create IvfFile if required.
     if (enable_save_bitstream) {
         std::string fn = std::get<0>(test_vector) + ".ivf";
-        output_file_ = new IvfFile(fn.c_str());
+        output_file_ = new IvfFile(fn);
     }
 
     ASSERT_NE(psnr_src_, nullptr) << "PSNR source create failed!";
@@ -366,14 +359,14 @@ void SvtAv1E2ETestFramework::output_stat() {
     }
 }
 
-void SvtAv1E2ETestFramework::gen_frame_event(EncTestSetting &setting,
+void SvtAv1E2ETestFramework::gen_frame_event(const EncTestSetting &setting,
                                              uint32_t frame_count,
                                              void **head) {
     EbPrivDataNode *node = nullptr;
-    for (TestFrameEvent event : setting.event_vector) {
+    for (const TestFrameEvent &event : setting.event_vector) {
         if (std::get<1>(event) == frame_count) {
             printf("%s param list:\t", std::get<0>(event).c_str());
-            for (std::string str : std::get<3>(event))
+            for (const std::string &str : std::get<3>(event))
                 printf("%s\t", str.c_str());
             printf("\n");
             EbPrivDataNode *new_node =
@@ -505,7 +498,7 @@ void SvtAv1E2ETestFramework::run_encode_process() {
             // try to get one encoded frame, flush the encoder
             // if src_file_eos is true
             do {
-                // non-blocking call
+                // non-blocking call if not using low-delay
                 EbBufferHeaderType *enc_out = nullptr;
                 {
                     TimeAutoCount counter(ENCODING, collect_);
@@ -519,7 +512,6 @@ void SvtAv1E2ETestFramework::run_encode_process() {
 
                 // process the output buffer
                 if (return_error != EB_NoErrorEmptyQueue && enc_out) {
-#if OPT_LD_LATENCY2
                     if (enc_out->flags & EB_BUFFERFLAG_EOS) {
                         enc_file_eos = true;
                         printf("Encoder EOS\n");
@@ -528,15 +520,6 @@ void SvtAv1E2ETestFramework::run_encode_process() {
                         TimeAutoCount counter(CONFORMANCE, collect_);
                         process_compress_data(enc_out);
                     }
-#else
-                    // send to reference decoder
-                    TimeAutoCount counter(CONFORMANCE, collect_);
-                    process_compress_data(enc_out);
-                    if (enc_out->flags & EB_BUFFERFLAG_EOS) {
-                        enc_file_eos = true;
-                        printf("Encoder EOS\n");
-                    }
-#endif
                     // check if the process has encounter error, break out if
                     // true, like the recon frame does not match with decoded
                     // frame.
@@ -752,8 +735,7 @@ void SvtAv1E2ETestFramework::get_recon_frame(const SvtAv1Context &ctxt,
         ASSERT_NE(new_frame->buffer, nullptr)
             << "can not get new buffer of recon frame!!";
 
-        EbBufferHeaderType recon_frame;
-        memset(&recon_frame, 0, sizeof(recon_frame));
+        EbBufferHeaderType recon_frame{};
         recon_frame.size = sizeof(EbBufferHeaderType);
         recon_frame.p_buffer = new_frame->buffer;
         recon_frame.n_alloc_len = new_frame->buf_size;
@@ -804,7 +786,7 @@ void SvtAv1E2ETestFramework::get_recon_frame(const SvtAv1Context &ctxt,
     } while (true);
 }
 
-SvtAv1E2ETestFramework::IvfFile::IvfFile(std::string path) {
+SvtAv1E2ETestFramework::IvfFile::IvfFile(const std::string &path) {
     FOPEN(file, path.c_str(), "wb");
     byte_count_since_ivf = 0;
     ivf_count = 0;
